#ifndef MPIB_GETOPT_H_
#define MPIB_GETOPT_H_

/*!
 * \defgroup getopt Options for executables
 * \details As getopt cannot be reused, this module provides an interface for getopt.
 * \code
MPIB_getopt_x_default(&x);
MPIB_getopt_y_default(&y);
if (root == 0)
{
	char options[256] = "";
	strcat(options, MPIB_getopt_x_options());
	strcat(options, MPIB_getopt_y_options());
	while ((c = getopt(argc, argv, options)) != -1)
	{
		MPIB_getopt_x_optarg(c, &x);
		MPIB_getopt_y_optarg(c, &y);
	}
}
MPIB_getopt_x_bcast(&x, 0, MPI_COMM_WORLD);
MPIB_getopt_y_bcast(&y, 0, MPI_COMM_WORLD);
 * \endcode
 * 
 * Typical parameters of executables are as follows:
 * - \b -h help
 * - \b -O \e S collective operation (required):\n
			MPI_Scatter, MPIB_Scatter_linear, MPIB_Scatter_binomial\n
			MPI_Gather, MPIB_Gather_linear, MPIB_Gather_binomial\n
			MPI_Scatterv\n
			MPI_Gatherv\n
 * - \b -t \e S timing: max, root, global (default: max)
 * - \b -s \e I message size stride (default: 1024)
 * - \b -m \e I maximum message size (default: 204800)
 * - \b -p 0/1 parallel p2p benchmarking (default: 1)
 * - \b -r \e I minimum number of repetitions (default: 5)
 * - \b -R \e I maximum number of repetitions (default: 100)
 * - \b -c \e D confidence level: 0 < D < 1 (default: 0.95)
 * - \b -e \e D error: 0 < D < 1 (default: 0.025)
 * 
 * where:
 * - \e S - string
 * - \e I - integer
 * - \e D - double
 * 
 * All sets of parameters are managed by the following functions:
 * - \b MPIB_getopt_x_default - fills the parameters by default values
 * - \b MPIB_getopt_x_options - returns a string of the getopt options
 * - \b MPIB_getopt_x_optarg - fills the parameters by the getopt argument
 * - \b MPIB_getopt_x_bcast - broadcasts the parameters (parallel)
 * \{
 */

void MPIB_getopt_help_default(int* exit);
const char* MPIB_getopt_help_options();
void MPIB_getopt_help_optarg(int c, const char* program, int* exit);
void MPIB_getopt_help_bcast(int* exit, int root, MPI_Comm comm);

void MPIB_getopt_precision_default(MPIB_precision* precision);
const char* MPIB_getopt_precision_options();
void MPIB_getopt_precision_optarg(int c, MPIB_precision* precision);
void MPIB_getopt_precision_bcast(MPIB_precision* precision, int root, MPI_Comm comm);

void MPIB_getopt_msg_default(int* stride, int* max_size);
const char* MPIB_getopt_msg_options();
void MPIB_getopt_msg_optarg(int c, int* stride, int* max_size);
void MPIB_getopt_msg_bcast(int* stride, int* max_size, int root, MPI_Comm comm);

void MPIB_getopt_coll_default(char* operation, char* timing);
const char* MPIB_getopt_coll_options();
void MPIB_getopt_coll_optarg(int c, char* operation, char* timing);
void MPIB_getopt_coll_bcast(char* operation, MPIB_coll_container** container,
	char* timing, MPIB_measure_coll* measure, int root, MPI_Comm comm);

void MPIB_getopt_p2p_default(int* parallel);
const char* MPIB_getopt_p2p_options();
void MPIB_getopt_p2p_optarg(int c, int* parallel);
void MPIB_getopt_p2p_bcast(int* parallel, int root, MPI_Comm comm);

/*!
 * \}
 */

#endif /*MPIB_GETOPT_H_*/
